'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const storage_1 = require("./services/storage");
const config_1 = require("./services/config");
const scanner_1 = require("./services/scanner");
const resolver_1 = require("./providers/resolver");
const stream_1 = require("./providers/stream");
const pathUtils = require("./utils/paths");
const fsUtils = require("./utils/fs");
function assertInput(directory, language) {
    if (!directory) {
        throw new TypeError('directory must be a string');
    }
    const type = typeof language;
    if (!language || (type !== 'string' && type !== 'object')) {
        throw new TypeError('language must be a string or an object');
    }
    if (!fsUtils.pathExistsSync(directory)) {
        throw new Error('directory must exist');
    }
}
function setup(root, language, options) {
    assertInput(root, language);
    const storage = new storage_1.StorageService();
    options = Object.assign({
        snapshot: {},
        cleanupInterval: null,
        log: () => console.log,
        vinylFile: false,
        basedir: null
    }, options);
    options.scanner = Object.assign({
        depth: 30,
        exclude: ['.git', '**/node_modules', '**/bower_components']
    }, options.scanner);
    // Loading data if provided dependency tree
    if (options.snapshot) {
        storage.load(options.snapshot);
    }
    // Run invalidation
    if (options.cleanupInterval) {
        storage.startInvalidation(options.cleanupInterval * 1000);
    }
    // Expanding of Glob-patterns that should be excluded during scanning
    if (options.scanner.exclude) {
        options.scanner.exclude = pathUtils.expandGlobPatterns(options.scanner.exclude);
    }
    root = pathUtils.normalize(root);
    const configService = new config_1.ConfigService(language);
    const scannerService = new scanner_1.ScannerService(root, storage, configService.getConfig(), options);
    const resolverProvider = new resolver_1.ResolverProvider(storage);
    const streamProvider = new stream_1.StreamProvider(root, storage, configService.getConfig(), options);
    return {
        storage: () => storage.snapshot(),
        keys: () => storage.keys(),
        load: (snapshot) => storage.load(snapshot),
        scan: (filepath, stats) => scannerService.scan(filepath, stats),
        resolver: resolverProvider,
        stream: (filepath, stats) => streamProvider.run(filepath, stats),
        filter: (filepath) => streamProvider.filter(filepath)
    };
}
exports.setup = setup;
