'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const path = require("path");
const through2 = require("through2");
const Vinyl = require("vinyl");
const scanner_1 = require("../services/scanner");
const resolver_1 = require("../providers/resolver");
const pathUtils = require("../utils/paths");
const fsUtils = require("../utils/fs");
class StreamProvider {
    constructor(root, storage, language, options) {
        this.root = root;
        this.storage = storage;
        this.language = language;
        this.options = options;
        this.scanner = new scanner_1.ScannerService(this.root, this.storage, this.language, this.options);
        this.resolver = new resolver_1.ResolverProvider(this.storage);
    }
    /**
     * Starts scanning the directory and push Vinyl file to a Stream if it is required.
     */
    run(filepath, stats) {
        const self = this;
        // Protection against undefined
        if (typeof filepath !== 'string') {
            filepath = null;
            stats = null;
        }
        // Protection against WIN32 paths
        filepath = pathUtils.normalize(filepath);
        return through2.obj(function (file, _enc, cb) {
            const mainFile = self.makeMainFilePath(self.root, file);
            // Update Storage
            self.scanner.scan(filepath, stats).then((lastChangedFile) => {
                // Protection against bad paths
                if (!filepath && !lastChangedFile) {
                    self.pushFile(this, file, mainFile);
                    return cb();
                }
                filepath = filepath ? filepath : lastChangedFile;
                self.filterFileByDependencies(filepath, mainFile, this, file, cb);
            }).catch(cb);
        });
    }
    /**
     * Push Vinyl file to a Stream if it is required.
     */
    filter(filepath) {
        const self = this;
        return through2.obj(function (file, _enc, cb) {
            const mainFile = self.makeMainFilePath(self.root, file);
            // Protection against bad paths
            if (!filepath) {
                self.pushFile(this, file, mainFile);
                return cb();
            }
            self.filterFileByDependencies(filepath, mainFile, this, file, cb);
        });
    }
    /**
     * Determines whether to send the Vinyl file to a Stream.
     */
    filterFileByDependencies(filepath, mainFile, streamCtx, file, cb) {
        const changedFile = pathUtils.normalize(filepath);
        if (this.resolver.checkDependency(mainFile, changedFile)) {
            if (this.options.makeVinylFile) {
                return this.makeVinylFile(mainFile).then((vFile) => {
                    this.pushFile(streamCtx, vFile, mainFile);
                    return cb();
                });
            }
            this.pushFile(streamCtx, file, mainFile);
            return cb();
        }
        return cb();
    }
    /**
     * Push Vinyl file to a Stream.
     */
    pushFile(ctx, file, filepath) {
        ctx.push(file);
        this.options.log(filepath);
    }
    /**
     * Calculates relative path of the Vinyl file in Stream.
     */
    makeMainFilePath(root, file) {
        let filepath = '';
        if (file.path) {
            filepath = pathUtils.relative(file.cwd, file.path);
        }
        if (!filepath.startsWith(root)) {
            filepath = pathUtils.join(root, filepath);
        }
        return pathUtils.normalize(filepath);
    }
    /**
     * Creates Vinyl File for filepath.
     */
    makeVinylFile(filepath) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const exists = yield fsUtils.pathExists(filepath);
            if (!exists) {
                return null;
            }
            const stat = yield fsUtils.statFile(filepath);
            const content = yield fsUtils.readFile(filepath);
            const fullpath = pathUtils.join(process.cwd(), filepath);
            return new Vinyl({
                base: path.dirname(fullpath),
                path: fullpath,
                contents: new Buffer(content),
                stat
            });
        });
    }
}
exports.StreamProvider = StreamProvider;
